################################################################################
# R code to accompany Rankins et al. -                                         #
# Sociality modulates nutritional carrying capacity of an endangered species   #
# Written by Seth Rankins                                                      #
# Warning this code takes multiple days to run.                                #
# This code requires a computer with at least 3 cores to run.                  #
# Note the "artistic" plot in the paper was not created using written computer #
# code, so this script does not reproduce it.                                  #
# The local time is used as the seed for the analyses. Thus, there might be    #
# slight discrepancies between the results between runs due to random number   #
# generation in the analyses.                                                  #
################################################################################

# Load packages that I use here
library(brms)
library(cplm)
library(bayestestR)
library(ggeffects)
library(cowplot)
library(ggplot2)

# What is the relationship between slope and biomass?
#-------------------------------------------------------------------------------
# load in data
# This is a random sample of 100,000 rows of the entire data set. It was created
# by using:
# df_sub <- dplyr::sample_n(dataframe, 100000)
# I did this to speed up computation time (which is still slow).
# The results from the sub-sample and full data set are the same.
df_SlopeBiomass <- read.table("RandomSampleSlopeBiomass.txt",
                              sep = ",",
                              header =TRUE)

# Specify model
out <- bcplm(Biomass ~ Slope, data = df_sub, n.iter = 110000,
              n.burnin = 10000, n.thin = 10)

# Look at model summary
summary(out)
summary(gelman.diag(out$sims.list)[[1]][, 1])

# Check trace plots and unimodality
plot(out$sims.list[, c(1:2)])

# Get probability of direction
describe_posterior(unlist(out2$sims.list[, 2]), centrality = "mean",
                   test = c("p_direction"))

#-------------------------------------------------------------------------------
# Is Group size related to population size?
#-------------------------------------------------------------------------------
# Load Data 
df_GSPop <- read.table("GroupSizePop.txt", sep = ",", header =TRUE)

# Specify model
GSPop <- brm(GroupSize ~ NumberFemales + (1|Herd) + (1|Year), data = df_GSPop, 
             family = negbinomial(link = "log"),
             iter = 105000,
             chains = 3,
             cores = 3,
             warmup = 5000,
             thin = 10,
             prior = c(set_prior("normal(0, 10^6)",
                                 class = "b",
                                 coef= "NumberFemales"),
                       set_prior("normal(0, 10^6)",
                                 class= "Intercept"),
                       set_prior("normal(0, 10^6)",
                                 class= "sd",
                                 coef = "Intercept",
                                 group = "Herd"),
                       set_prior("normal(0, 10^6)",
                                 class= "sd",
                                 coef = "Intercept",
                                 group = "Year")))

# Look at model summary
print(summary(GSPop), digits = 5)

# Check trace plots and unimodality
plot(GSPop, ask = FALSE)

# Get probability of direction
describe_posterior(posterior_samples(GSPop)$b_NumberFemales,
                   centrality = "mean",
                   test = c("p_direction"))

# Estimate the biological "effect" size
round(exp(1.76089 + 0.00939*min(df_GSPop$NumberFemales)), 0)
round(exp(1.76089 + 0.00939*max(df_GSPop$NumberFemales)), 0)

#-------------------------------------------------------------------------------
# What is the relationship between group size and habitat use?
#-------------------------------------------------------------------------------
# Load data
df_GSHabitat <- read.table("GroupSizeUsedHabitat.txt", sep = ",", header =TRUE)

# Specify model 2 models
m1 <- brm(Slope ~ GroupSize + (1|AnimalID),
          data = df_GSHabitat, 
          family = gaussian(),
          iter = 105000,
          chains = 3,
          cores = 3,
          warmup = 5000,
          thin = 10,
          prior = c(set_prior("normal(0, 10^6)",
                              class = "b",
                              coef= "GroupSize"),
                    set_prior("normal(0, 10^6)",
                              class= "Intercept"),
                    set_prior("normal(0, 10^6)",
                              class= "sd",
                              coef = "Intercept",
                              group = "AnimalID"),
                    set_prior("normal(0, 10^6)",
                              class= "sigma")))

m2 <- bcplm(Biomass ~ GroupSize + (1|AnimalID),
            data = df_GSHabitat,
            n.iter = 105000,
            n.burnin = 5000,
            n.thin = 10)

# Look at model summaries
print(summary(m1), digits = 5)
summary(m2)
summary(gelman.diag(m2$sims.list)[[1]][, 1])

# Look at trace plots and check for unimodality
plot(m1, ask = FALSE)
plot(m2$sims.list[, c(1:2)])

# Get probability of direction
describe_posterior(posterior_samples(m1)$b_GroupSize, centrality = "mean",
                   test = c("p_direction"))
describe_posterior(unlist(m2$sims.list[, 2]), centrality = "mean",
                   test = c("p_direction"))

# Look at biological "effect" size
# m1
32.31221 - 0.02294*min(df_GSHabitat$GroupSize)
32.31221 - 0.02294*max(df_GSHabitat$GroupSize)
# m2
exp(3.631936 + 0.004905*min(data2$GroupSize))
exp(3.631936 + 0.004905*max(data2$GroupSize))

# Now see what the percent change in nutritional carrying capacity is
df <- read.table("SlopeBiomass.txt", sep = ",", header =TRUE)
round((sum(subset(df, df$Slope >= 31.05051)$Biomass)/sum(subset(df, df$Slope >= 32.28927)$Biomass)*100 - 100), 0)



# Get the 95% credible intervals and predicted mean for plotting
plot_df1 <- ggemmeans(m1, terms = "GroupSize")
plot_df2 <- data.frame(X = seq(from = min(df_GSHabitat$GroupSize),
                               to = max(df_GSHabitat$GroupSize),
                               by = 2), Predict = NA_real_,
                       upr = NA_real_, lwr = NA_real_)

for(i in 1:33){
  vec <- seq(from = min(df_GSHabitat$GroupSize),
             to = max(df_GSHabitat$GroupSize), by = 2)
  plot_df2$Predict[i] <- mean(exp(unlist(m2$sims.list[, 1]) + unlist(m2$sims.list[, 2])*vec[i]))
  plot_df2$upr[i] <- hdi(exp(unlist(m2$sims.list[, 1]) + unlist(m2$sims.list[, 2])*vec[i]), ci = 0.95)$CI_high
  plot_df2$lwr[i] <- hdi(exp(unlist(m2$sims.list[, 1]) + unlist(m2$sims.list[, 2])*vec[i]), ci = 0.95)$CI_low
}

# Create plot
scaleFactor <- 0.5

ggplot() +
  geom_line(data = plot_df1, aes(x, predicted), size = 1.1, col = "black") +
  geom_ribbon(data = plot_df1, aes(x = x, ymin = conf.low, ymax = conf.high), fill = "#B9BDB8", alpha = 0.5) +
  geom_line(data = plot_df2, aes(X, Predict*scaleFactor), size = 1.1, col = "#1D8641") +
  geom_ribbon(data = plot_df2, aes(x = X, ymin = lwr*scaleFactor, ymax = upr*scaleFactor), fill = "#1D8641", alpha = 0.2) +
  theme_classic() +
  xlab("Group Size") +
  theme(text = element_text(size = 20)) +
  
  scale_y_continuous(name = "Terrain Steepness (Slope in Degrees)", sec.axis = sec_axis(~./scaleFactor, name = "Forage Biomass (kg/ha)")) +
  theme(
    axis.title.y.left=element_text(color="black"),
    axis.text.y.left=element_text(color="black"),
    axis.title.y.right=element_text(color="#1D8641"),
    axis.text.y.right=element_text(color="#1D8641")
  ) +
  scale_x_continuous(breaks = c(0, 10, 20, 30, 40, 50, 60, 70),
                     limits = c(0, 70)) 

#-------------------------------------------------------------------------------
# What is the relationship between population size and habitat selection?
#-------------------------------------------------------------------------------
# Load data
df_RSF <- read.table("GroupSizeRSF.txt", sep = ",", header =TRUE)

# Split into winter and summer
df_summer <- subset(df_RSF, df_RSF$Season == "summer")
df_winter <- subset(df_RSF, df_RSF$Season == "winter")

# Specify the 4 models
mod1 <- brm(exp(ScaledSlope) ~ NumberFemales + AvalabiltySlope,
            data = df_winter,
            family = gaussian(),
            iter = 105000,
            chains = 3,
            cores = 3,
            warmup = 5000,
            thin = 10,
            prior = c(set_prior("normal(0, 10^6)",
                                class = "b",
                                coef= "NumberFemales"),
                      set_prior("normal(0, 10^6)",
                                class = "b",
                                coef = "AvalabiltySlope"),
                      set_prior("normal(0, 10^6)",
                                class= "Intercept"),
                      set_prior("normal(0, 10^6)",
                                class= "sigma")))

mod2 <- brm(exp(ScaledBiomass) ~ NumberFemales + AvalabiltyBiomass,
            data = df_winter,
            family = gaussian(),
            iter = 105000,
            chains = 3,
            cores = 3,
            warmup = 5000,
            thin = 10,
            prior = c(set_prior("normal(0, 10^6)",
                                class = "b",
                                coef= "NumberFemales"),
                      set_prior("normal(0, 10^6)",
                                class = "b",
                                coef = "AvalabiltyBiomass"),
                      set_prior("normal(0, 10^6)",
                                class= "Intercept"),
                      set_prior("normal(0, 10^6)",
                                class= "sigma")))

mod3 <- brm(exp(ScaledSlope) ~ NumberFemales + AvalabiltySlope,
            data = df_summer,
            family = gaussian(),
            iter = 105000,
            chains = 3,
            cores = 3,
            warmup = 5000,
            thin = 10,
            prior = c(set_prior("normal(0, 10^6)",
                                class = "b",
                                coef= "NumberFemales"),
                      set_prior("normal(0, 10^6)",
                                class = "b",
                                coef = "AvalabiltySlope"),
                      set_prior("normal(0, 10^6)",
                                class= "Intercept"),
                      set_prior("normal(0, 10^6)",
                                class= "sigma")))

mod4 <- brm(exp(ScaledBiomass) ~ NumberFemales + AvalabiltyBiomass,
            data = df_summer,
            family = gaussian(),
            iter = 105000,
            chains = 3,
            cores = 3,
            warmup = 5000,
            thin = 10,
            prior = c(set_prior("normal(0, 10^6)",
                                class = "b",
                                coef= "NumberFemales"),
                      set_prior("normal(0, 10^6)",
                                class = "b",
                                coef = "AvalabiltyBiomass"),
                      set_prior("normal(0, 10^6)",
                                class= "Intercept"),
                      set_prior("normal(0, 10^6)",
                                class= "sigma")))

# Look at model summaries
print(summary(mod1), digits = 5)
print(summary(mod2), digits = 5)
print(summary(mod3), digits = 5)
print(summary(mod4), digits = 5)

# Look at trace plots and check for unimodality
plot(mod1, ask = FALSE)
plot(mod2, ask = FALSE)
plot(mod3, ask = FALSE)
plot(mod4, ask = FALSE)

# Get probability of direction
describe_posterior(posterior_samples(mod1)$b_NumberFemales, centrality = "mean",
                   test = c("p_direction"))
describe_posterior(posterior_samples(mod2)$b_NumberFemales, centrality = "mean",
                   test = c("p_direction"))
describe_posterior(posterior_samples(mod3)$b_NumberFemales, centrality = "mean",
                   test = c("p_direction"))
describe_posterior(posterior_samples(mod4)$b_NumberFemales, centrality = "mean",
                   test = c("p_direction"))

# Get the 95% credible intervals and predicted mean for plotting
df_mod1 <- ggemmeans(mod1, terms = "NumberFemales")
df_mod2 <- ggemmeans(mod2, terms = "NumberFemales")
df_mod3 <- ggemmeans(mod3, terms = "NumberFemales")
df_mod4 <- ggemmeans(mod4, terms = "NumberFemales")

# Create separate plots
p1 <- ggplot() +
  geom_line(data = df_mod1, aes(x, predicted, color = "black"), size = 1.1) +
  geom_ribbon(data = df_mod1, aes(x = x, ymin = conf.low, ymax = conf.high), fill = "#B9BDB8", alpha = 0.2) +
  geom_line(data = df_mod2, aes(x, predicted, color = "#1D8641"), size = 1.1) +
  geom_ribbon(data = df_mod2, aes(x = x, ymin = conf.low, ymax = conf.high), fill = "#1D8641", alpha = 0.2) +
  theme_classic() +
  xlab("") +
  theme(text = element_text(size = 20)) +
  scale_x_continuous(limits = c(0, 60),
                     expand = c(0, 0.5)) +
  theme(axis.text.x = element_blank(),
        axis.ticks.x = element_blank()) +
  
  ylab(paste("  Relative Selection \n Strength")) +
  scale_y_continuous(breaks = c(0, 0.5, 1, 1.5, 2),
                     limits = c(0, 2),
                     expand = c(0, 0)) +
  scale_color_identity(guide = "legend", labels = c("Forage Biomass", "Terrain Steepness"), name = "") +
  theme(legend.position = c(0.5, 0.95),
        legend.text=element_text(size = 20))

p2 <- ggplot() +
  geom_line(data = df_mod3, aes(x, predicted, color = "black"), size = 1.1) +
  geom_ribbon(data = df_mod3, aes(x = x, ymin = conf.low, ymax = conf.high), fill = "#B9BDB8", alpha = 0.2) +
  geom_line(data = df_mod4, aes(x, predicted, color = "#1D8641"), size = 1.1) +
  geom_ribbon(data = df_mod4, aes(x = x, ymin = conf.low, ymax = conf.high), fill = "#1D8641", alpha = 0.2) +
  theme_classic() +
  xlab("") +
  theme(text = element_text(size = 20)) +
  ylab(paste("  Relative Selection \n Strength")) +
  scale_y_continuous(breaks = c(0, 0.5, 1, 1.5, 2),
                     limits = c(0, 2),
                     expand = c(0, 0)) +
  scale_color_identity(guide = "legend", labels = c("", ""), name = "") +
  theme(legend.position = c(-5, -5)) +
  scale_x_continuous(breaks = c(0, 10, 20, 30, 40, 50, 60),
                     limits = c(0, 60),
                     expand = c(0, 0.5)) +
  xlab("Minimum Count of Female Bighorn Sheep (Proxy for Population Size)")

# Create a multi-panel plot
plot_grid(
  p1, p2,
  labels = "AUTO", ncol = 1,
  label_size = 30,
  vjust = c(1, 0),
  align = "v")
